import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/log_entry.dart';
import 'package:diameter/models/log_event.dart';
import 'package:diameter/screens/log/log_entry/log_event_detail.dart';
import 'package:diameter/utils/date_time_utils.dart';
import 'package:flutter/material.dart';

class LogEventListScreen extends StatefulWidget {
  final LogEntry? logEntry;
  final List<LogEvent> logEvents;
  final List<LogEvent> activeEvents;
  final Function()? reload;

  const LogEventListScreen(
      {Key? key, this.logEntry, this.logEvents = const [], this.activeEvents = const [], this.reload})
      : super(key: key);

  @override
  _LogEventListScreenState createState() => _LogEventListScreenState();
}

class _LogEventListScreenState extends State<LogEventListScreen> {
  void reload({String? message}) {
    if (widget.reload != null) {
      widget.reload!();
    }

    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void handleEditAction(LogEvent event) {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => LogEventDetailScreen(
          logEntryId: widget.logEntry?.id ?? 0,
          id: event.id,
        ),
      ),
    ).then((message) => reload(message: message));
  }

  void onDelete(LogEvent logEvent) {
    LogEvent.remove(logEvent.id);
    reload(message: 'Event deleted');
  }

  void handleDeleteAction(LogEvent logEvent) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(logEvent),
        message: 'Are you sure you want to delete this Event?',
      );
    } else {
      onDelete(logEvent);
    }
  }

  void onStop(LogEvent event) async {
    event.endTime = DateTime.now();
    event.endLogEntry.target =
        LogEntry.get(widget.logEntry?.id ?? 0) ?? LogEntry(time: DateTime.now());
    LogEvent.put(event);
    reload(message: 'Event ended');
  }

  void handleStopAction(LogEvent event) async {
    if (showConfirmationDialogOnStopEvent) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onStop(event),
        message: 'Are you sure you want to end this Event?',
        confirmationLabel: 'END EVENT',
      );
    } else {
      onStop(event);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        Expanded(
          child: (widget.logEvents.isNotEmpty ||
                  widget.activeEvents.isNotEmpty)
              ? ListView.builder(
                  shrinkWrap: true,
                  itemCount: widget.logEvents.length +
                      widget.activeEvents.length,
                  itemBuilder: (context, index) {
                    final event = (widget.logEvents +
                      widget.activeEvents)[index];
                    return ListTile(
                      onTap: () {
                        handleEditAction(event);
                      },
                      title: Row(
                        mainAxisSize: MainAxisSize.max,
                        children: [
                          Expanded(
                              child: Text(event.eventType.target?.value ?? '')),
                        ],
                      ),
                      subtitle: Text(
                          '${DateTimeUtils.displayDateTime(event.time)}${event.hasEndTime ? ' - ${DateTimeUtils.displayDateTime(event.endTime, fallback: '(ongoing)')}' : ''}'),
                      trailing: Row(
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          event.hasEndTime && event.endTime == null ? IconButton(
                            icon: const Icon(
                              Icons.stop,
                              color: Colors.blue,
                            ),
                            onPressed: () => handleStopAction(event),
                          ) : Container(),
                          const SizedBox(width: 24),
                          IconButton(
                            icon: const Icon(
                              Icons.delete,
                              color: Colors.blue,
                            ),
                            onPressed: () => handleDeleteAction(event),
                          ),
                        ],
                      ),
                    );
                  })
              : Center(
                  child: Text(widget.logEntry == null ? 'There are no active Events!'
                      : 'You have not added any Events to this Log Entry yet!'),
                ),
        ),
      ],
    );
  }
}
